/**
 * EasyBeans
 * Copyright (C) 2011 Bull S.A.S.
 * Contact: easybeans@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: AbsDeployerList.java 6062 2011-11-16 01:10:08Z cazauxj $
 * --------------------------------------------------------------------------
 */
package org.ow2.util.ee.deploy.impl.deployer;

import org.ow2.util.ee.deploy.api.deployable.IDeployable;
import org.ow2.util.ee.deploy.api.deployer.DeployerException;
import org.ow2.util.ee.deploy.api.deployer.IDeployer;
import org.ow2.util.ee.deploy.api.deployer.UnsupportedDeployerException;
import org.ow2.util.log.Log;
import org.ow2.util.log.LogFactory;

import java.util.ArrayList;
import java.util.List;

/**
 * Abstract class that defines common methods to deploy a list of deployable.
 * @author Jeremy Cazaux
 */
public abstract class AbsDeployerList<T extends IDeployable<T>> implements IDeployer<T> {

    /**
     * Logger.
     */
    private Log logger = LogFactory.getLog(AbsDeployerList.class);

    /**
     * {@inheritDoc}
     */
    public abstract void deploy(IDeployable<T> deployable) throws DeployerException, UnsupportedDeployerException;

    /**
     * {@inheritDoc}
     */
    public abstract void undeploy(IDeployable<T> deployable) throws DeployerException;

    /**
     * {@inheritDoc}
     */
    public abstract boolean isDeployed(IDeployable<T> deployable) throws DeployerException;

    /**
     * {@inheritDoc}
     */
    public abstract boolean supports(IDeployable<?> deployable);

    /**
     * {@inheritDoc}
     */
    public List<IDeployable<T>> deploy(List<IDeployable<T>> deployables) throws UnsupportedDeployerException {
        List<IDeployable<T>> failed = new ArrayList<IDeployable<T>>();

        for (IDeployable<T> deployable: deployables) {
            try {
                deploy(deployable);
            } catch (DeployerException e) {
                logger.error("Cannot deploy the deployable " + deployable.getArchive().getName() + ".", e);
                failed.add(deployable);
            }
        }

        return failed;
    }

    /**
     * {@inheritDoc}
     */
    public List<IDeployable<T>> undeploy(final List<IDeployable<T>> deployables) {
        List<IDeployable<T>> failed = new ArrayList<IDeployable<T>>();

        for (IDeployable<T> deployable: deployables) {
            try {
                undeploy(deployable);
            } catch (DeployerException e) {
                logger.error("Cannot undeploy the deployable " + deployable.getArchive().getName() + ".", e);
                failed.add(deployable);
            }
        }

        return failed;
    }

    /**
     * {@inheritDoc}
     */
    public boolean supports(List<IDeployable<?>> deployables) {

        for (IDeployable deployable: deployables) {
            if (!supports(deployable)) {
                return false;
            }
        }
        return true;
    }
}
